bl_info = {
    "name": "Chalk Style",
    "author": "Chipp Walters",
    "version": (1, 0, 13),
    "blender": (2, 82, 0),
    "location": "View3D",
    "description": "Creates a wireframe ambient occlusion render",
    "warning": "",
    "wiki_url": "https://docs.google.com/document/d/1G6Lv0NZzd5U5aeZQXXt_w-whkUxDkfHq0kCMzaq1svI/edit?usp=sharing",
    "category": "Render",
}

import bpy

def button01(context):
    # SAVES PREV RENDER SETTINGS, THEN SETS RENDER TYPE, THEN RELOADS SAVED SETTINGS
    C = bpy.context

    # IMAGE SETTINGS
    scene = C.scene

    settings = scene.render.image_settings
    old_ff_settings = {}
    for attr in dir(scene.render.ffmpeg):
        if attr not in ['__doc__', '__module__', '__slots__', 'bl_rna', 'rna_type']:
            old_ff_settings[attr] = getattr(scene.render.ffmpeg, attr)

    old_format = settings.file_format
    old_quality = settings.quality

    settings.file_format = 'JPEG'
    settings.quality = 90
    
    old_folderpath = scene.render.filepath
       
    # RENDER SETTINGS
    old_resolution_x = scene.render.resolution_x
    old_resolution_y = scene.render.resolution_y
#    old_renderpercent = scene.render.resolution_percentage
    
    old_displaydevice = bpy.context.scene.display_settings.display_device
    bpy.context.scene.display_settings.display_device = 'sRGB'
    
    old_viewtransform = bpy.context.scene.view_settings.view_transform
    bpy.context.scene.view_settings.view_transform = 'Standard'
    
    old_look = bpy.context.scene.view_settings.look
    bpy.context.scene.view_settings.look = 'None'
    
    old_exposure = bpy.context.scene.view_settings.exposure
    bpy.context.scene.view_settings.exposure = 0
    
    old_gamma = bpy.context.scene.view_settings.gamma
    bpy.context.scene.view_settings.gamma = 1

        
    # CONFIGURE OUTLINE SCENE
    old_shadingtype = bpy.context.space_data.shading.type
    bpy.context.space_data.shading.type = 'SOLID'
    
    old_shadinglight = bpy.context.space_data.shading.light
    bpy.context.space_data.shading.light = 'FLAT'
    
    old_showcavity = bpy.context.space_data.shading.show_cavity
    bpy.context.space_data.shading.show_cavity = False

    old_showshadows = bpy.context.space_data.shading.show_shadows    
    bpy.context.space_data.shading.show_shadows = False
    
    old_showxray = bpy.context.space_data.shading.show_xray
    bpy.context.space_data.shading.show_xray = False
    
    old_backtype = bpy.context.space_data.shading.background_type
    bpy.context.space_data.shading.background_type = 'VIEWPORT'
    
    old_backcolor = bpy.context.space_data.shading.background_color
    bpy.context.space_data.shading.background_color = (1, 1, 1)
    
    old_colortype = bpy.context.space_data.shading.color_type
    bpy.context.space_data.shading.color_type = 'SINGLE'
    
    old_singlecolor = bpy.context.space_data.shading.single_color
    bpy.context.space_data.shading.single_color = (1, 1, 1)
    
    old_showoverlays = bpy.context.space_data.overlay.show_overlays
    bpy.context.space_data.overlay.show_overlays = True
    
    old_show_wf = bpy.context.space_data.overlay.show_wireframes
    bpy.context.space_data.overlay.show_wireframes = False
    
    old_showgrid = bpy.context.space_data.overlay.show_ortho_grid
    bpy.context.space_data.overlay.show_ortho_grid = False
    
    old_showoutline = bpy.context.space_data.shading.show_object_outline
    bpy.context.space_data.shading.show_object_outline = True
    

    # NEED 2X RESOLUTION FOR OUTLINE AND WIREFRAME MODES
    new_rez_x = 2 * old_resolution_x
    new_rez_y = 2 * old_resolution_y
    scene.render.resolution_x = new_rez_x
    scene.render.resolution_y = new_rez_y
    # ***don't need to account for percentage <> 100***
    #scene.render.resolution_percentage = 100

    # RENDER VIEWPORT
    new_renderpath = old_folderpath + 'outline.png'
    scene.render.filepath = new_renderpath
    scene.render.image_settings.file_format = 'PNG'
    bpy.ops.render.opengl(write_still=True, view_context=True)
    
    # CONFIGURE WIREFRAME SCENE
    bpy.context.space_data.shading.type = 'WIREFRAME'
    
    old_showxraywf = bpy.context.space_data.shading.show_xray_wireframe
    bpy.context.space_data.shading.show_xray_wireframe = False

    # RENDER VIEWPORT
    new_renderpath = old_folderpath + 'wireframe.png'
    scene.render.filepath = new_renderpath
    bpy.ops.render.opengl(write_still=True, view_context=True)
    
    # CONFIGURE AO PASS
    old_aotype = bpy.context.space_data.shading.type
    bpy.context.space_data.shading.type = 'MATERIAL'
    
    old_aorenderpass = bpy.context.space_data.shading.render_pass
    bpy.context.space_data.shading.render_pass = 'AO'

    old_usegtao = bpy.context.scene.eevee.use_gtao
    bpy.context.scene.eevee.use_gtao = True
    
    old_gtaodistance = bpy.context.scene.eevee.gtao_distance
    bpy.context.scene.eevee.gtao_distance = 1
    
    old_gtaofactor = bpy.context.scene.eevee.gtao_factor
    bpy.context.scene.eevee.gtao_factor = 1.5
    
    # SET RESOLUTION TO ORIGINAL
    scene.render.resolution_x = old_resolution_x
    scene.render.resolution_y = old_resolution_y
#    scene.render.resolution_percentage = old_renderpercent

    # RENDER VIEWPORT
    new_renderpath = old_folderpath + 'ao.png'
    scene.render.filepath = new_renderpath
    bpy.ops.render.opengl(write_still=True, view_context=True)


    # RESET EVERYTHING BACK TO THE WAY IT WAS
    settings.file_format = old_format
    settings.quality = old_quality
    
    scene.render.filepath =  old_folderpath
    
    bpy.context.space_data.shading.light = old_shadinglight
    bpy.context.space_data.shading.show_cavity = old_showcavity
    bpy.context.space_data.shading.show_shadows = old_showshadows
    bpy.context.space_data.shading.show_xray = old_showxray
    bpy.context.space_data.shading.background_type = old_backtype
    bpy.context.space_data.shading.background_color = old_backcolor
    bpy.context.space_data.shading.color_type = old_colortype
    bpy.context.space_data.shading.single_color = old_singlecolor
    bpy.context.space_data.overlay.show_overlays = old_showoverlays
    bpy.context.space_data.shading.show_xray_wireframe = old_showxraywf
    bpy.context.space_data.overlay.show_wireframes = old_show_wf
    bpy.context.space_data.overlay.show_ortho_grid = old_showgrid
    bpy.context.space_data.shading.show_object_outline = old_showoutline

    
    bpy.context.space_data.shading.type = old_aotype
    if old_aorenderpass == "":
        old_aorenderpass = "None"
    bpy.context.space_data.shading.render_pass = old_aorenderpass
    bpy.context.scene.eevee.use_gtao = old_usegtao
    bpy.context.scene.eevee.gtao_distance = old_gtaodistance
    bpy.context.scene.eevee.gtao_factor = old_gtaofactor
    bpy.context.space_data.shading.type = old_shadingtype
    
    bpy.context.scene.display_settings.display_device = old_displaydevice
    bpy.context.scene.view_settings.view_transform = old_viewtransform
    if old_look == "":
        old_look = "None"
    bpy.context.scene.view_settings.look = old_look
    bpy.context.scene.view_settings.exposure = old_exposure
    bpy.context.scene.view_settings.gamma = old_gamma

    for key in old_ff_settings:
        setattr(scene.render.ffmpeg, key, old_ff_settings[key])

class RenderThree(bpy.types.Operator):
#    """Press to """
    bl_idname = "chipp.render_three"
    bl_label = "Render CHALK STYLE"
    bl_description = (
        "Be sure and set the folderpath\n"
        "in the output properties panel")


    def execute(self, context):
        button01(context)
        return {'FINISHED'}

class Help_Operator(bpy.types.Operator):					     					
    bl_idname  = "object.chalk_style_help"						
    bl_label   = "Go to Page With Documentation"	
    bl_description = "Link to online documentation"				
    bl_options = {'REGISTER', 'UNDO'}  					
    
    def execute(self,context):		
        url="docs.google.com/document/d/1G6Lv0NZzd5U5aeZQXXt_w-whkUxDkfHq0kCMzaq1svI/edit?usp=sharing"					
        try:
            webbrowser.open(url)
        except Exception as e:
            bpy.ops.wm.url_open(url=url)
        return {'FINISHED'}	

class LayoutMainPanel(bpy.types.Panel):
    """  """
    bl_idname = "TSCENE_PT_layout"
    bl_label = "Chalk Style"
    bl_category = "Chalk Style"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"

    def draw(self, context):
        layout = self.layout

        scene = context.scene
       
        # Big render button
        #layout.label(text="Press to create 3 render files")
        row = layout.row()
        row.scale_y = 1.5
        row.operator("chipp.render_three", icon="NODE_COMPOSITING")
        row = layout.row()
        row = layout.row()
        row.scale_y = 1.0
        row.operator("object.chalk_style_help", text="Documentation", icon= "TEXT")
        #row = layout.row()


class LayoutChildPanel_2(bpy.types.Panel):
    """  """
    bl_idname = "TSCENE_PT3_layout"
    bl_label = "Overlays"
    bl_category = "ChalkStyle"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_parent_id = "TSCENE_PT_layout"
#    bl_options = {'DEFAULT_CLOSED'}


    def draw(self, context):
        layout = self.layout

        scene = context.scene
        
        row = layout.row()
        
        # OVERLAY CHECKBOXS
        row.prop(bpy.context.space_data.overlay, "show_extras", text = 'Show Extras')
        row = layout.row()
        row.prop(bpy.context.space_data.overlay, "show_floor", text = 'Show Floor')
        row = layout.row()
        row.prop(bpy.context.space_data.overlay, "show_ortho_grid", text = 'Show Grid')
        row = layout.row()
        row.prop(bpy.context.space_data.overlay, "show_axis_x", text = 'Show X axis')
        row = layout.row()
        row.prop(bpy.context.space_data.overlay, "show_axis_y", text = 'Show Y axis')
        row = layout.row()
        row.prop(bpy.context.space_data.overlay, "show_axis_z", text = 'Show Z axis')
        row = layout.row()
        row.prop(bpy.context.space_data.overlay, "show_cursor", text = 'Show Cursor')
        row = layout.row()
        row.prop(bpy.context.space_data.overlay, "show_object_origins", text = 'Show Object Origins')
        row = layout.row()
        row.prop(bpy.context.space_data.overlay, "show_outline_selected", text = 'Show Outline Selected')
        row = layout.row()
        row.prop(bpy.context.space_data.overlay, "show_relationship_lines", text = 'Show Relationship Lines')
        row = layout.row()
        row.prop(bpy.context.space_data.overlay, "show_bones", text = 'Show Bones')
        row = layout.row()
        row.prop(bpy.context.space_data.overlay, "show_motion_paths", text = 'Show Motion Paths')
        row = layout.row()
                

class LayoutChildPanel_1(bpy.types.Panel):
    """Creates a Panel in the scene context of the properties editor"""
    bl_idname = "TSCENE_PT2_layout"
    bl_label = "Object types"
    bl_category = "Chalk Style"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_parent_id = "TSCENE_PT_layout"
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout

        scene = context.scene
        
        row = layout.row()
 
        # OBJECT TYPES CHECKBOXS
        row = layout.row()
        row.prop(bpy.context.space_data, "show_object_viewport_curve", text = 'Show Curves')
        row = layout.row()
        row.prop(bpy.context.space_data, "show_object_viewport_armature", text = 'Show Armatures')
        row = layout.row()
        row.prop(bpy.context.space_data, "show_object_viewport_lattice", text = 'Show Lattices')
        row = layout.row()
        row.prop(bpy.context.space_data, "show_object_viewport_light", text = 'Show Lights')
        row = layout.row()
        row.prop(bpy.context.space_data, "show_object_viewport_light_probe", text = 'Show Light Probes')
        row = layout.row()
        row.prop(bpy.context.space_data, "show_object_viewport_camera", text = 'Show Camera')
        row = layout.row()

         


def register():
    bpy.utils.register_class(LayoutMainPanel)
    bpy.utils.register_class(LayoutChildPanel_2)
    bpy.utils.register_class(LayoutChildPanel_1)
    bpy.utils.register_class(Help_Operator)
    bpy.utils.register_class(RenderThree)
    

def unregister():
    bpy.utils.unregister_class(LayoutMainPanel)
    bpy.utils.unregister_class(LayoutChildPanel_2)
    bpy.utils.unregister_class(LayoutChildPanel_1)
    bpy.utils.unregister_class(Help_Operator)
    bpy.utils.unregister_class(RenderThree)

if __name__ == "__main__":
    register()
